from . import Config


def load_with_extends(path, key='extends', root=None):
    """
    Helper for the :ref:`"extends" <extends-pattern>` pattern.

    :param path:
      The path of the configuration file to start with.
    :param key:
      The key to use to indicate that another file should be used as a base.
    :param root:
      If supplied, configuration is extracted from this key at the root of
      each configuration file that is loaded, provided it is present. If missing
      from any file, the whole configuration from that file is used instead.
    """
    configs = []
    while path:
        config = Config.from_path(path)
        if root is not None and root in config:
            config = config[root]
        configs.append(config)
        path = config.get(key)
    config = Config()
    for layer in reversed(configs):
        config.merge(layer)
    config.data.pop(key, None)
    return config



from .config import Config
from .merge import default_mergers
from .mapping import source, target, convert, required, if_supplied, value

__all__ = (
    'Config',
    'source',
    'target',
    'convert',
    'required',
    'if_supplied',
    'value',
    'default_mergers',
)



import json

from testfixtures import compare

from configurator import Config
from configurator.patterns import load_with_extends


class TestLoadWithExtends(object):

    def test_key_not_present(self, dir):
        path = dir.write('file.json', '{"key":"value"}')
        compare(load_with_extends(path, key='extends'), expected=Config({'key': 'value'}))

    def test_root(self, dir):
        path1 = dir.write('file1.json', '{"root": {"f1key":"f1value"}}')
        path2 = dir.write('file2.json', json.dumps({'root': {
            'f2key': 'f2value',
            'extends': path1,
        }}))
        compare(load_with_extends(path2, key='extends', root='root'), expected=Config({
            'f1key': 'k1value',
            'f2key': 'f2value',
        }))
